<?php
if (!defined('DC_RC_PATH')) { return; }

class OpenIDDCAuth extends dcAuth
{

	public function __construct(&$core)
	{
		if (isset($_GET['janrain_nonce']) || !empty($_POST['openid']))
		{
			// trompe "auth.php" pour qu'il appelle la fonction checkUser
			$_POST['user_id'] = '__openid';
			$_POST['user_pwd'] = '__openid';
			$this->auth_back = isset($_GET['janrain_nonce']);
		}
		parent::__construct($core);
		$core->addBehavior('loginPageHTMLHead',array('OpenIDDCAuth','echoOpenIDCSS'));
	}

	/**
	This method builds a canonical OpenID from a string.
	
	@param openid <b>string</b>		OpenID in any form (http can be missing, can be uppercase, ...)
	@return <b>string</b> The canonical form of given OpenID
	*/
	public static function cleanOpenID($openid)
	{
		$openid = strtolower($openid);
		if (!$openid)
		{
			return $openid;
		}
		$parts = parse_url($openid);
		if (empty($parts['scheme']))
		{
			$parts['scheme'] = 'http';
		}
		$openid = $parts['scheme'].'://';
		if (!empty($parts['user']))
		{
			$user = $parts['user'];
			if (!empty($parts['pass']))
			{
				$user .= ':'.$parts['pass'];
			}
			$openid .= $user.'@';
		}
		if (empty($parts['host']) && !empty($parts['path']))
		{
			$parts['host'] = $parts['path'];
			$parts['path'] = '/';
		}
		$openid .= $parts['host'];
		if (!empty($parts['port']))
		{
			$openid .= ':'.$parts['port'];
		}
		if (empty($parts['path']))
		{
			$parts['path'] = '/';
		}
		$openid .= $parts['path'];
		if (!empty($parts['query']))
		{
			$openid .= '?'.$parts['query'];
		}
		if (!empty($parts['fragment']))
		{
			$openid .= '#'.$parts['fragment'];
		}
		return $openid;
	}
	
	/**
	This method retreives a user linked with a given OpenID.
	
	@param openid <b>string</b>		OpenID of user (will be cleaned)
	@return <b>record</b> The row from user_table that has this openid
	*/
	protected function getUserWithOpenID($openid)
	{
		$openid = OpenIDDCAuth::cleanOpenID($openid);
		
		$rec = $this->con->select('SELECT * FROM '.$this->user_table.' WHERE user_options LIKE "%s:6:\"openid\";s:'.strlen($openid).':\"'.$this->con->db_escape_string($openid).'\"%"');
		
		if ($rec->count() != 1) {
			return false;
		}
		$rec->moveStart();
		
		return $rec;
	}

	/**
	This site url for OpenID authentication. If left at null, it will be set automatically
	but it reduces security. If you have several blogs (using different urls) keep it null
	*/
	protected $site_url = null;
	
	/**
	True only when coming back from OpenID authentication
	*/
	protected $auth_back = false;

	/**
	This method uses OpenID library to create an OpenID consumer. It uses dotclear's cache
	directory and DB sessions.
	*/
	protected function createOpenIDConsumer()
	{
		require_once 'Auth/OpenID/Consumer.php';
		require_once 'Auth/OpenID/FileStore.php';
		$this->core->session->start();
		
		$store = new Auth_OpenID_FileStore(DC_TPL_CACHE .'/oid_store');

		$consumer = new Auth_OpenID_Consumer($store);
		
		// make sure site_url is not null
		$this->initSiteUrl();
		
		return $consumer;
	}
	
	/**
	This method computes site_url from $_SERVER vars if it is not set yet.
	*/
	protected function initSiteUrl()
	{
		if (!isset($this->site_url))
		{
			$this->site_url = 'http';
			if (!empty($_SERVER['HTTPS'])) $this->site_url .= 's';
			$this->site_url .= '://';
			$this->site_url .= $_SERVER['HTTP_HOST'];
			if (empty($_SERVER['HTTPS']) && $_SERVER['SERVER_PORT'] != 80 || !empty($_SERVER['HTTPS']) && $_SERVER['SERVER_PORT'] != 443)
				$this->site_url .= ':'.$_SERVER['SERVER_PORT'];
			$this->site_url .= $_SERVER['REQUEST_URI'];
		}
	}
	
	## dcAuth overrides
	
	/**
	This method checks if user session cookie exists
	
	@return	<b>boolean</b>
	*/
	public function sessionExists()
	{
		// when we come back from openid auth, the session was used and if we
		// go in parent::checkSession we might loose everything
		return !$this->auth_back && parent::sessionExists();
	}
	
	/**
	This method computes the authentication form for OpenID
	
	@param user_id <b>string</b>	id of the user the last time he/she tried to login
	@return <b>string</b>	an HTML string containng the inside of the form to display
	*/
	public function authForm($user_id)
	{
		$echo = '<fieldset>'.
		'<p class="onlyopenid"><label>'.__('OpenID:').
		form::field(array('openid'),30,255,'','openid',1).'</label></p>'.
		
		'<p class="notopenid"><label>'.__('Login:').' '.
		form::field(array('user_id'),20,32,html::escapeHTML($user_id),'',1).'</label></p>'.
		
		'<p class="notopenid"><label>'.__('Password:').' '.
		form::password(array('user_pwd'),20,255,'','',2).'</label></p>'.
		
		'<p><label class="classic">'.
		form::checkbox(array('user_remember'),1,'','',3).' '.
		__('Remember my ID on this computer').'</label></p>'.
		
		'<script type="text/javascript">
			function openid_auth_oldmethod() {
				$(".notopenid").hide();
				$("#login-screen").css("width",350);
				$(".onlyopenid").show();
				$("input[@name=openid]").css("width",320)[0].focus();
			}
		</script>'.
		
		'<p><input type="submit" value="'.__('login').'" tabindex="4" /> <input class="notopenid" type="button" value="'.__('OpenID').'" onclick="openid_auth_oldmethod()" /></p>';
		
		if (!empty($_REQUEST['blog'])) {
			$echo .= form::hidden('blog',$_REQUEST['blog']);
		}
		
		$echo .=
		'</fieldset>'.
		
		'<p class="notopenid">'.__('You must accept cookies in order to use the private area.').'</p>';
		
		if (parent::allowPassChange()) {
			$echo .= '<p class="notopenid"><a href="auth.php?recover=1">'.__('I forgot my password').'</a></p>';
		}
		
		$echo .= '<script type="text/javascript">$(".onlyopenid").hide();</script>';
		return $echo;
	}
	
	/// @name Credentials and user permissions
	//@{
	/**
	Checks if user exists and can log in. <var>$pwd</var> argument is optionnal
	while you may need to check user without password. This method will create
	credentials and populate all needed object properties.
	
	@param	user_id	<b>string</b>		User ID
	@param	pwd		<b>string</b>		User password
	@param	user_key	<b>string</b>		User key check
	@return	<b>boolean</b>
	*/
	public function checkUser($user_id, $pwd=null, $user_key=null)
	{
		// handles user_remember
		if (!empty($user_key)) {
			// if __openid is used as password, we try to autoconnect using OpenID
			if (http::browserUID(DC_MASTER_KEY.$user_id.crypt::hmac(DC_MASTER_KEY,'__openid')) == $user_key) {
				// let's try relogging using openid
				$rs = $this->con->select(
					'SELECT user_options '.
					'FROM '.$this->con->escapeSystem($this->user_table).' '.
					"WHERE user_id = '".$this->con->escape($user_id)."' ");
				if ($rs->isEmpty()) {
					// no user found with this user_id
					return false;
				}
				$rs->extend('rsExtUser');
				$user_options = $rs->options();
				if (empty($user_options['openid'])) {
					// this user doesn't have any OpenID in his profile
					return false;
				}
				$_POST['openid'] = $user_options['openid'];
			}
		}
		
		// handles login from auth form
		if (!empty($_POST['openid']))
		{
			$consumer = $this->createOpenIDConsumer();
			$auth = $consumer->begin(OpenIDDCAuth::cleanOpenID($_POST['openid']));
			if ($auth) {
				$url = $auth->redirectURL($this->site_url, $this->site_url);
				// save this post vars in session to retreive in auth_back
				if (!empty($_POST['user_remember'])) {
					$_SESSION['user_remember'] = $_POST['user_remember'];
				}
				if (!empty($_POST['blog'])) {
					$_SESSION['blog'] = $_POST['blog'];
				}
				// redirect to matching OpenID server
				http::redirect($url);
				die();
			}
			// unable to create OpenID auth, id may not be a valid OpenID
			global $user_id;
			$user_id = '';
			return false;
		}
		
		// handles auth back from OpenID server
		if ($this->auth_back)
		{
			$consumer = $this->createOpenIDConsumer();
			$response = $consumer->complete($this->site_url);
			// retreive post vars from session
			if (!empty($_SESSION['user_remember'])) {
				$_POST['user_remember'] = $_SESSION['user_remember'];
				unset($_SESSION['user_remember']);
			}
			if (!empty($_SESSION['blog'])) {
				$_POST['blog'] = $_SESSION['blog'];
				unset($_SESSION['blog']);
			}
			if ($response->status == Auth_OpenID_SUCCESS) {
				// distant auth was a success
				$user = $this->getUserWithOpenID($response->identity_url);
				if ($user !== null)
				{
					// there is a user having this OpenID in his/her profile
					global $user_id, $user_pwd;
					$user_id = $user->user_id;
					$user_pwd = '__openid';
					return parent::checkUser($user_id);
				}
			}
			// distant auth didn't go very well
			global $user_id;
			$user_id = '';
			return false;
		}
		
		// falls back on basic auth
		return parent::checkUser($user_id,null,$user_key);

	}
	
	## Behaviors to update user's OpenID in admin/preferences
	
	/**
	This method will add elements to the form on admin page "preferences.php"
	
	@param core <b>dcCore</b>		DotClear core
	*/
	public static function adminPreferencesForm(&$core)
	{
		echo
		'<fieldset>'.
		'<legend>'.__('OpenID').'</legend>'.
		'<p><label>'.__('User OpenID:').form::field('user_openid',50,255,$core->auth->getOption('openid'),'openid').'</label></p>'.
		'</fieldset>';
	}
	
	/**
	This method will add elements to the form on admin page "user.php"
	
	@param rs <b>record</b>		user's record
	*/
	public static function adminUserForm(&$rs)
	{
		$user_options = $rs->options();
		echo
		'<fieldset>'.
		'<legend>'.__('OpenID').'</legend>'.
		'<p><label>'.__('User OpenID:').form::field('user_openid',50,255,$user_options['openid'],'openid').'</label></p>'.
		'</fieldset>';
	}
	
	/**
	This method will update user's parameters and options before updating
	in database
	
	@param cur <b>cursor</b>		Cursor on user
	@param user_id <b>integer</b>	Id of user
	*/
	public static function adminBeforeUserUpdate($cur, $user_id)
	{
		$user_options = get_object_vars($cur->user_options);
		$user_options['openid'] = OpenIDDCAuth::cleanOpenID($_POST['user_openid']);
		$cur->user_options = new ArrayObject($user_options);
	}
	
	/**
	This method will add a CSS link to add style for OpenID input fields
	*/
	public static function addOpenIDCSS()
	{
		return '<link rel="stylesheet" type="text/css" href="index.php?pf=authopenid/openid.css" />';
	}
	
	/**
	This method will add a CSS link to add style for OpenID input fields
	*/
	public static function echoOpenIDCSS()
	{
		echo OpenIDDCAuth::addOpenIDCSS();
	}
	
}
